"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.any = void 0;
exports.anyOf = anyOf;
exports.charClass = charClass;
exports.charRange = charRange;
exports.inverted = exports.digit = void 0;
exports.negated = negated;
exports.word = exports.whitespace = exports.notWord = exports.notWhitespace = exports.notDigit = exports.nonWord = exports.nonWhitespace = exports.nonDigit = void 0;
/**
 * Character range from start to end (inclusive).
 */

const any = exports.any = {
  type: 'characterClass',
  escape: '.',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
const digit = exports.digit = {
  type: 'characterClass',
  escape: '\\d',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
const nonDigit = exports.nonDigit = {
  type: 'characterClass',
  escape: '\\D',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
const word = exports.word = {
  type: 'characterClass',
  escape: '\\w',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
const nonWord = exports.nonWord = {
  type: 'characterClass',
  escape: '\\W',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
const whitespace = exports.whitespace = {
  type: 'characterClass',
  escape: '\\s',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};
const nonWhitespace = exports.nonWhitespace = {
  type: 'characterClass',
  escape: '\\S',
  chars: [],
  ranges: [],
  isNegated: false,
  encode: encodeCharacterClass
};

/**
 * @deprecated Renamed to `nonDigit`.
 */
const notDigit = exports.notDigit = nonDigit;

/**
 * @deprecated Renamed to `nonWord`.
 */
const notWord = exports.notWord = nonWord;

/**
 * @deprecated Renamed to `nonWhitespace`.
 */
const notWhitespace = exports.notWhitespace = nonWhitespace;
function charClass(...elements) {
  if (elements.some(e => e.isNegated)) {
    throw new Error('`charClass` should receive only non-negated character classes');
  }
  if (elements.length === 1) {
    return elements[0];
  }
  return {
    type: 'characterClass',
    chars: elements.map(c => getAllChars(c)).flat(),
    ranges: elements.map(c => c.ranges).flat(),
    isNegated: false,
    encode: encodeCharacterClass
  };
}
function charRange(start, end) {
  if (start.length !== 1) {
    throw new Error('`charRange` should receive only single character `start` string');
  }
  if (end.length !== 1) {
    throw new Error('`charRange` should receive only single character `end` string');
  }
  if (start > end) {
    throw new Error('`start` should be before or equal to `end`');
  }
  return {
    type: 'characterClass',
    chars: [],
    ranges: [{
      start,
      end
    }],
    isNegated: false,
    encode: encodeCharacterClass
  };
}
function anyOf(characters) {
  const chars = characters.split('').map(c => escapeForCharacterClass(c));
  if (chars.length === 0) {
    throw new Error('`anyOf` should received at least one character');
  }
  return {
    type: 'characterClass',
    chars,
    ranges: [],
    isNegated: false,
    encode: encodeCharacterClass
  };
}
function negated(element) {
  return {
    type: 'characterClass',
    chars: element.chars,
    ranges: element.ranges,
    isNegated: !element.isNegated,
    encode: encodeCharacterClass
  };
}

/**
 * @deprecated Renamed to `negated`.
 */
const inverted = exports.inverted = negated;
function encodeCharacterClass() {
  if (this.escape === undefined && this.chars.length === 0 && this.ranges.length === 0) {
    throw new Error('Character class should contain at least one character or character range');
  }

  // Direct rendering for single-character class
  if (this.escape !== undefined && !this.chars.length && !this.ranges.length && !this.isNegated) {
    return {
      precedence: 'atom',
      pattern: this.escape
    };
  }
  const allChars = getAllChars(this);

  // If passed characters includes hyphen (`-`) it need to be moved to
  // first (or last) place in order to treat it as hyphen character and not a range.
  // See: https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_expressions/Character_classes#types
  const hyphen = allChars.includes('-') ? '-' : '';
  const caret = allChars.includes('^') ? '^' : '';
  const otherChars = allChars.filter(c => c !== '-' && c !== '^').join('');
  const ranges = this.ranges.map(({
    start,
    end
  }) => `${start}-${end}`).join('');
  const negation = this.isNegated ? '^' : '';
  let pattern = `[${negation}${ranges}${otherChars}${caret}${hyphen}]`;
  if (pattern === '[^-]') pattern = '[\\^-]';
  return {
    precedence: 'atom',
    pattern
  };
}
function escapeForCharacterClass(text) {
  return text.replace(/[\]\\]/g, '\\$&'); // $& means the whole matched string
}
function getAllChars(characterClass) {
  if (characterClass.escape === undefined) {
    return characterClass.chars;
  }
  return [characterClass.escape, ...characterClass.chars];
}
//# sourceMappingURL=character-class.js.map